//+------------------------------------------------------------------+
//|                                               Moving Average.mq4 |
//|                      Copyright  2005, MetaQuotes Software Corp. |
//|                                       http://www.metaquotes.net/ |
//+------------------------------------------------------------------+
#define MAGICMA  20050610

extern int Counter               = 1;
extern int TestsNumber           = 100;
extern double Lots               = 0.1;
extern double MaximumRisk        = 0.02;
extern double DecreaseFactor     = 3;
extern double MovingPeriod       = 12;
extern double MovingShift        = 6;
extern double MovingPeriodLow    = 12;
extern double MovingShiftLow     = 6;
extern double MovingPeriodStep   = 1;
extern double MovingShiftStep    = 1;

int TstsCnt;
int FilePtr;
double Criterion;
int h;
double Data[100][4];
double StartBalance;
double MaxEqu;
double MaxDD;
//+------------------------------------------------------------------+
//| expert initialization function                                   |
//+------------------------------------------------------------------+
int init()
  {
//----
   if (IsTesting() && TestsNumber > 0) {

     if (GlobalVariableCheck("TestsCnt")==true) {
       TstsCnt = GlobalVariableGet("TestsCnt"); 
     } else {
       TstsCnt = 0; 
       GlobalVariableSet("TestsCnt",0); 
     }
     if (GlobalVariableCheck("FilePtr")) {
       FilePtr = GlobalVariableGet("FilePtr"); 
     } else {
       FilePtr = 0; 
       GlobalVariableSet("FilePtr",0); 
     }
     
     MovingPeriod = MovingPeriodLow+(TstsCnt/10)*MovingPeriodStep;
     MovingShift = MovingShift+(TstsCnt%10)*MovingShiftStep;
   }   
   StartBalance = AccountBalance();
   MaxEqu = 0;
   MaxDD = 0;
//----
   return(0);
  }
//+------------------------------------------------------------------+
//| expert deinitialization function                                 |
//+------------------------------------------------------------------+
int deinit()
  {
//----
   if (AccountEquity() > MaxEqu) MaxEqu = AccountEquity();
   if (MaxEqu-AccountEquity() > MaxDD) MaxDD = MaxEqu-AccountEquity();
   Criterion = (AccountBalance()-StartBalance)/MaxDD;

   if (IsTesting() && TestsNumber > 0) {
     TstsCnt++;
     GlobalVariableSet("TestsCnt",TstsCnt); 
     if (TstsCnt == 1) {
       h=FileOpen("test.txt",FILE_CSV|FILE_WRITE,';');
       FileWrite(h,Criterion,MovingPeriod,MovingShift,TstsCnt);
       FilePtr = FileTell(h); 
       GlobalVariableSet("FilePtr",FilePtr);
       FileClose(h); 
     } else {
       h=FileOpen("test.txt",FILE_CSV|FILE_READ|FILE_WRITE,';');
       FileSeek(h,FilePtr, SEEK_SET);
       FileWrite(h,Criterion,MovingPeriod,MovingShift,TstsCnt);
       FilePtr = FileTell(h); 
       GlobalVariableSet("FilePtr",FilePtr);
       FileClose(h); 
       if (TstsCnt == TestsNumber) {
         h=FileOpen("test.txt",FILE_CSV|FILE_READ,';');
         int i = 0;
         while (i<100 && FileIsEnding(h)== false) {
           for (int j=0;j<4;j++) Data[i][j]=FileReadNumber(h); 
           i++;
         } 
         FileClose(h); 
         ArraySort(Data,WHOLE_ARRAY,0,MODE_DESCEND);
         h=FileOpen("test.txt",FILE_CSV|FILE_WRITE,';');
         for (i=0;i<100;i++) FileWrite(h,Data[i][0],Data[i][1],Data[i][2],Data[i][3]);
         FileClose(h); 
         GlobalVariableDel("TestsCnt"); 
         GlobalVariableDel("FilePtr");
       }
     }
   }
   
//----
   return(0);
  }
//+------------------------------------------------------------------+
//| Calculate open positions                                         |
//+------------------------------------------------------------------+
int CalculateCurrentOrders(string symbol)
  {
   int buys=0,sells=0;
//----
   for(int i=0;i<OrdersTotal();i++)
     {
      if(OrderSelect(i,SELECT_BY_POS,MODE_TRADES)==false) break;
      if(OrderSymbol()==Symbol() && OrderMagicNumber()==MAGICMA)
        {
         if(OrderType()==OP_BUY)  buys++;
         if(OrderType()==OP_SELL) sells++;
        }
     }
//---- return orders volume
   if(buys>0) return(buys);
   else       return(-sells);
  }
//+------------------------------------------------------------------+
//| Calculate optimal lot size                                       |
//+------------------------------------------------------------------+
double LotsOptimized()
  {
   double lot=Lots;
   int    orders=HistoryTotal();     // history orders total
   int    losses=0;                  // number of losses orders without a break
//---- select lot size
   lot=NormalizeDouble(AccountFreeMargin()*MaximumRisk/1000.0,1);
//---- calcuulate number of losses orders without a break
   if(DecreaseFactor>0)
     {
      for(int i=orders-1;i>=0;i--)
        {
         if(OrderSelect(i,SELECT_BY_POS,MODE_HISTORY)==false) { Print("Error in history!"); break; }
         if(OrderSymbol()!=Symbol() || OrderType()>OP_SELL) continue;
         //----
         if(OrderProfit()>0) break;
         if(OrderProfit()<0) losses++;
        }
      if(losses>1) lot=NormalizeDouble(lot-lot*losses/DecreaseFactor,1);
     }
//---- return lot size
   if(lot<0.1) lot=0.1;
   return(lot);
  }
//+------------------------------------------------------------------+
//| Check for open order conditions                                  |
//+------------------------------------------------------------------+
void CheckForOpen()
  {
   double ma;
   int    res;
//---- go trading only for first tiks of new bar
   if(Volume[0]>1) return;
//---- get Moving Average 
   ma=iMA(NULL,0,MovingPeriod,MovingShift,MODE_SMA,PRICE_CLOSE,0);
//---- sell conditions
   if(Open[1]>ma && Close[1]<ma)  
     {
      res=OrderSend(Symbol(),OP_SELL,LotsOptimized(),Bid,3,0,0,"",MAGICMA,0,Red);
      return;
     }
//---- buy conditions
   if(Open[1]<ma && Close[1]>ma)  
     {
      res=OrderSend(Symbol(),OP_BUY,LotsOptimized(),Ask,3,0,0,"",MAGICMA,0,Blue);
      return;
     }
//----
  }
//+------------------------------------------------------------------+
//| Check for close order conditions                                 |
//+------------------------------------------------------------------+
void CheckForClose()
  {
   double ma;
//---- go trading only for first tiks of new bar
   if(Volume[0]>1) return;
//---- get Moving Average 
   ma=iMA(NULL,0,MovingPeriod,MovingShift,MODE_SMA,PRICE_CLOSE,0);
//----
   for(int i=0;i<OrdersTotal();i++)
     {
      if(OrderSelect(i,SELECT_BY_POS,MODE_TRADES)==false)        break;
      if(OrderMagicNumber()!=MAGICMA || OrderSymbol()!=Symbol()) continue;
      //---- check order type 
      if(OrderType()==OP_BUY)
        {
         if(Open[1]>ma && Close[1]<ma) OrderClose(OrderTicket(),OrderLots(),Bid,3,White);
         break;
        }
      if(OrderType()==OP_SELL)
        {
         if(Open[1]<ma && Close[1]>ma) OrderClose(OrderTicket(),OrderLots(),Ask,3,White);
         break;
        }
     }
//----
  }
//+------------------------------------------------------------------+
//| Start function                                                   |
//+------------------------------------------------------------------+
void start()
  {
   if (AccountEquity() > MaxEqu) MaxEqu = AccountEquity();
   if (MaxEqu-AccountEquity() > MaxDD) MaxDD = MaxEqu-AccountEquity();
//---- check for history and trading
   if(Bars<100 || IsTradeAllowed()==false) return;
//---- calculate open orders by current symbol
   if(CalculateCurrentOrders(Symbol())==0) CheckForOpen();
   else                                    CheckForClose();
//----
  }
//+------------------------------------------------------------------+